PounceCam = LibStub("AceAddon-3.0"):NewAddon("PounceCam", "AceEvent-3.0", "AceTimer-3.0")

--==== upvalues ====--
local _G, PounceCam, LibStub, format, select = _G, PounceCam, LibStub, format, select
local GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID = GetCVar, SetCVar, GetSpellInfo, UnitClass, UnitGUID
--==================--


local CEvent = { module = PounceCam }
CEvent.__index = CEvent
function CEvent:Release()
	if self.handler then 
		self.module:UnregisterEvent(self.eventName)
		self.handler = nil
	end
end
function CEvent:Set(handler)
	if handler == self.handler then return end
	self:Release()
	self.module:RegisterEvent(self.eventName, handler)
	self.handler = handler
end
local Event_CLEU = setmetatable( { eventName = "COMBAT_LOG_EVENT_UNFILTERED" }, CEvent )

local strPlayerGUID
local iResetDelay
local strCameraFollowStyle
local strCameraFollowSpeed

local idSpellCatCharge = 49376
local idSpellCatSkullBash = 80965
local idSpellBearCharge = 16979
local idSpellBearSkullBash = 80964
local idSpellWarriorCharge = 100
local idSpellWarriorIntercept = 20252
local idSpellWarriorIntervene = 3411

local tblOptionsDefaults = {
	global = {
		CameraResetDelay          = 1,
		[idSpellCatCharge]        = true,
		[idSpellCatSkullBash]     = false,
		[idSpellBearCharge]       = true,
		[idSpellBearSkullBash]    = false,
		[idSpellWarriorCharge]    = true,
		[idSpellWarriorIntercept] = true,
		[idSpellWarriorIntervene] = true,
	}
}

local function CheckHandlerForDruid()
	local db = PounceCam.db.global
	if (db[idSpellCatCharge] or db[idSpellBearCharge] or db[idSpellCatSkullBash] or db[idSpellBearSkullBash]) then
		Event_CLEU:Set("CLEU")
	else
		Event_CLEU:Release()
	end
end

local function CheckHandlerForWarrior()
	local db = PounceCam.db.global
	if (db[idSpellWarriorCharge] or db[idSpellWarriorIntercept] or db[idSpellWarriorIntervene]) then
		Event_CLEU:Set("CLEU")
	else
		Event_CLEU:Release()
	end
end

local function GetOptionsUI(class)
	local function NameWithForm(id)
		local name, rank = GetSpellInfo(id)
		return format("%s (%s)", name, rank)
	end
	
	if (class == "DRUID") then
		return {
			name = "PounceCam",
			handler = PounceCam,
			type = 'group',
			childGroups = "tree",
			args = {
--[==[
				CtrlGroupCat = {
					order = 1,
					name = "Cat",
					handler = PounceCam,
					type = 'group',
					args = {
--]==]
						CtrlCatCharge = {
							order = 1,
							type = "toggle",
							name = NameWithForm(idSpellCatCharge),
							desc = NameWithForm(idSpellCatCharge),
							get = function(info) return PounceCam.db.global[idSpellCatCharge] end,
							set = function(info, value) 
								PounceCam.db.global[idSpellCatCharge] = value 
								CheckHandlerForDruid()
							end,
						},
						CtrlCatSkullBash = {
							order = 2,
							type = "toggle",
							name = NameWithForm(idSpellCatSkullBash),
							desc = NameWithForm(idSpellCatSkullBash),
							get = function(info) return PounceCam.db.global[idSpellCatSkullBash] end,
							set = function(info, value) 
								PounceCam.db.global[idSpellCatSkullBash] = value 
								CheckHandlerForDruid()
							end,
						},
						Padding_Cat = {
							order = 3,
							type = "description",
							name = "",
						},

--[==[
					},
				},
				CtrlGroupBear = {
					order = 2,
					name = "Bear",
					handler = PounceCam,
					type = 'group',
					args = {
--]==]
						CtrlBearCharge = {
							order = 4,
							type = "toggle",
							name = NameWithForm(idSpellBearCharge),
							desc = NameWithForm(idSpellBearCharge),
							get = function(info) return PounceCam.db.global[idSpellBearCharge] end,
							set = function(info, value) 
								PounceCam.db.global[idSpellBearCharge] = value 
								CheckHandlerForDruid()
							end,
						},
						CtrlBearSkullBash = {
							order = 5,
							type = "toggle",
							name = NameWithForm(idSpellBearSkullBash),
							desc = NameWithForm(idSpellBearSkullBash),
							get = function(info) return PounceCam.db.global[idSpellBearSkullBash] end,
							set = function(info, value) 
								PounceCam.db.global[idSpellBearSkullBash] = value 
								CheckHandlerForDruid()
							end,
						},
						Padding_Bear = {
							order = 6,
							type = "description",
							name = "",
						},
					},
--				},
--			},
		}

	elseif (class == "WARRIOR") then
		return {
			name = "PounceCam",
			handler = PounceCam,
			type = 'group',
			args = {
				CtrlWarriorCharge = {
					order = 1,
					type = "toggle",
					name = GetSpellInfo(idSpellWarriorCharge),
					desc = GetSpellInfo(idSpellWarriorCharge),
					get = function(info) return PounceCam.db.global[idSpellWarriorCharge] end,
					set = function(info, value) 
						PounceCam.db.global[idSpellWarriorCharge] = value 
						CheckHandlerForWarrior()
					end,
				},
				CtrlWarriorIntercept = {
					order = 2,
					type = "toggle",
					name = GetSpellInfo(idSpellWarriorIntercept),
					desc = GetSpellInfo(idSpellWarriorIntercept),
					get = function(info) return PounceCam.db.global[idSpellWarriorIntercept] end,
					set = function(info, value) 
						PounceCam.db.global[idSpellWarriorIntercept] = value 
						CheckHandlerForWarrior()
					end,
				},
				CtrlWarriorIntervene = {
					order = 2,
					type = "toggle",
					name = GetSpellInfo(idSpellWarriorIntervene),
					desc = GetSpellInfo(idSpellWarriorIntervene),
					get = function(info) return PounceCam.db.global[idSpellWarriorIntervene] end,
					set = function(info, value) 
						PounceCam.db.global[idSpellWarriorIntervene] = value 
						CheckHandlerForWarrior()
					end,
				},
			},
		}

	end
end


-- Called when the addon is loaded
function PounceCam:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PounceCamDB", tblOptionsDefaults)
end

-- Called when the addon is enabled
function PounceCam:OnEnable()
	local _,class = UnitClass("player")
	if (class == "DRUID" or class == "WARRIOR") then
		LibStub("AceConfig-3.0"):RegisterOptionsTable("PounceCam", GetOptionsUI(class))	
		self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PounceCam")

		iResetDelay = self.db.global.CameraResetDelay
		strPlayerGUID = UnitGUID("player")
		if (class == "DRUID") then
			CheckHandlerForDruid()
		elseif (class == "WARRIOR") then
			CheckHandlerForWarrior()
		end
	end
end

-- Called when the addon is Disabled
function PounceCam:OnDisable()
	Event_CLEU:Release()
end


function PounceCam:ResetCameraFollow()
	SetCVar("cameraSmoothStyle",    strCameraFollowStyle)
	SetCVar("cameraYawSmoothSpeed", strCameraFollowSpeed)
	-- print("resetting camera")
end

function PounceCam:CLEU(_, ...)
	local eventType = select(2, ...)
	local sourceGUID = select(4, ...)

	if(sourceGUID == strPlayerGUID and eventType == "SPELL_CAST_SUCCESS") then
		local spellId = select(12, ...)

		if (self.db.global[spellId]) then
			strCameraFollowStyle = GetCVar("cameraSmoothStyle")
			strCameraFollowSpeed = GetCVar("cameraYawSmoothSpeed")

			SetCVar("cameraSmoothStyle",    "3")
			SetCVar("cameraYawSmoothSpeed", "270")
			PounceCam:ScheduleTimer("ResetCameraFollow", iResetDelay)
		end	
	end
end
